#include <stdlib.h>
#include <packet.h>
#include <plugin.h>
#include <decode.h>

/* GRE: Generic routing encapsulation */
int gre_decode(struct packet *);
struct proto gre_p=init_proto("gre", gre_decode);
struct proto_req gre_r[]={
	proto_request("ipv4", 47),
	null_request()
};

int gre_decode(struct packet *p)
{
	struct proto_child *pc;
	struct layer *l=&p->layer[p->llen-1];
	
	if ( p->llen >= PKT_LAYERS ) return 0;
	
	if ( (p->layer[p->llen].h.raw=
		l->h.raw+sizeof(pkt_grehdr_t))
		>= p->end ) return 0;

	p->layer[p->llen].flags=0;
	for(pc=l->proto->children; pc; pc=pc->next)
	{
		if ( l->h.gre->proto == pc->id ) {
			p->layer[p->llen++].proto=pc->proto;
			return pc->proto->decode(p);
		}
	}

	p->layer[p->llen++].proto=NULL;
	return 0;
}

int _PLUGIN_INIT (struct plugin_in *in, struct plugin_out *out)
{
	proc_decode_add decode_add;
	int ok=0;
	
	plugin_check(in, out);
	
	PLUGIN_ID("decode.gre", "Generic Routing Encapsulation");
	PLUGIN_VERSION(2, 0);
	PLUGIN_AUTHOR("Gianni Tedesco", "gianni@ecsc.co.uk");
	PLUGIN_LICENSE("GPL");
	
	if ( !(decode_add=in->import("decode.add")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	ok+=decode_add(&gre_p, gre_r);
	
	return (ok) ? PLUGIN_ERR_OK : PLUGIN_ERR_FAIL;
}

int _PLUGIN_UNLOAD (int code) {
	return PLUGIN_ERR_OK;
}
