#ifndef __PACKET_HEADER_INCLUDED__
#define __PACKET_HEADER_INCLUDED__

#include <sys/time.h>
#include <sys/types.h>

/* packet macros */
#include <pmacros.h>

/* Various protocol headers */
#include <pkt/eth.h>
#include <pkt/ip.h>
#include <pkt/icmp.h>
#include <pkt/igmp.h>
#include <pkt/tcp.h>
#include <pkt/udp.h>
#include <pkt/arp.h>
#include <pkt/gre.h>
#include <pkt/sll.h>

/* Maximum number of layers in decode */
#define PKT_LAYERS	8

/* Maximum number of realms a packet can be in */
#define PKT_REALMS	8

typedef unsigned long long int serial_t;

struct layer {
	union {
		p_eth		eth;
		p_ip		ip;
		p_icmp		icmp;
		p_igmp		igmp;
		p_tcp		tcp;
		p_arp		arp;
		p_udp		udp;
		p_gre		gre;
		p_sll		sll;
		void		*raw;
	}h;
	struct proto	*proto;
	unsigned int	flags;
};

struct rlayer {
	struct realm		*realm;
	unsigned int		layer;
};

struct packet {
	/* Serial number for packet */
	serial_t	serial;
	
	/* length of packet on the wire */
	size_t		len;

	/* Length of captured packet */
	size_t		caplen;
	
	/* decode information */
	struct layer	layer[PKT_LAYERS];
	unsigned int	llen;

	/* Realms this packet belongs to */
	struct rlayer	realm[PKT_REALMS];
	unsigned int	rlen;
	unsigned int	present;

	/* Actual packet data */
	void		*base;
	void		*end;

	/* time of capture */
	struct timeval	time;

	/* Capture device that cought us */
	struct cap_s	*capture;

	/* Hand waving miscellany */
#define FP_CLONE	(1<<1) /* memcpy to clone the packet? */
#define FP_LIVE		(1<<2) /* Live capture */
	u_int32_t	flags;
};

/* preferred snaplen */
extern size_t snaplen;

/* subsystem API */
void packet_init(void);

#endif /* __PACKET_HEADER_INCLUDED__ */
